﻿using System;
using System.Net;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Threading.Tasks;
using Bing.Maps;

namespace BingOverview
{
	public static class Search
	{
		public static async Task<Response> GetSearchResponse(string key, string type, string query, Location location)
		{
			Uri queryRequest = new Uri(
				string.Format("https://api.datamarket.azure.com/Bing/Search/v1/Composite?Sources=%27{0}%27&Query=%27{1}%27&Latitude={2}&Longitude={3}&$top=20&$format=json",
				type, query, location.Latitude, location.Longitude));
			
			System.Net.Http.HttpClientHandler handler = new System.Net.Http.HttpClientHandler();
			handler.Credentials = new NetworkCredential(key, key);
			System.Net.Http.HttpClient client = new System.Net.Http.HttpClient(handler);

			var response = await client.GetAsync(queryRequest); 

			using (var stream = await response.Content.ReadAsStreamAsync())
			{
				DataContractJsonSerializer ser = new DataContractJsonSerializer(typeof(Response));
				return ser.ReadObject(stream) as Response;
			}
		}
	}

	[DataContract]
	public class Response
	{
		[DataMember(Name = "d")]
		public ResultSet ResultSet { get; set; }
	}

	[DataContract]
	public class ResultSet
	{
		[DataMember(Name = "__copyright")]
		public string Copyright { get; set; }

		[DataMember(Name = "results")]
		public ExpandableSearchResult[] Results { get; set; }
	}
	[DataContract]
	public class ExpandableSearchResult
	{
		[DataMember(Name = "ID")]
		public Guid ID { get; set; }

		[DataMember(Name = "AlteredQuery")]
		public string AlteredQuery { get; set; }

		[DataMember(Name = "AlterationOverrideQuery")]
		public string AlterationOverrideQuery { get; set; }

		[DataMember(Name = "Web")]
		public WebResult[] Web { get; set; }

		[DataMember(Name = "Image")]
		public ImageResult[] Image { get; set; }

		[DataMember(Name = "Video")]
		public VideoResult[] Video { get; set; }

		[DataMember(Name = "News")]
		public NewsResult[] News { get; set; }

		[DataMember(Name = "RelatedSearch")]
		public RelatedSearchResult[] RelatedSearch { get; set; }

		[DataMember(Name = "SpellingSuggestions")]
		public SpellResult[] SpellingSuggestions { get; set; }

	}

	[DataContract]
	public class WebResult
	{
		[DataMember(Name = "ID")]
		public Guid ID { get; set; }

		[DataMember(Name = "Title")]
		public string Title { get; set; }

		[DataMember(Name = "Description")]
		public string Description { get; set; }

		[DataMember(Name = "DisplayUrl")]
		public string DisplayUrl { get; set; }

		[DataMember(Name = "Url")]
		public string Url { get; set; }
	}

	[DataContract]
	public class ImageResult
	{
		[DataMember(Name = "ID")]
		public Guid ID { get; set; }

		[DataMember(Name = "Title")]
		public string Title { get; set; }

		[DataMember(Name = "MediaUrl")]
		public string MediaUrl { get; set; }

		[DataMember(Name = "SourceUrl")]
		public string SourceUrl { get; set; }

		[DataMember(Name = "DisplayUrl")]
		public string DisplayUrl { get; set; }

		[DataMember(Name = "Width")]
		public int Width { get; set; }

		[DataMember(Name = "Height")]
		public int Height { get; set; }

		[DataMember(Name = "FileSize")]
		public long FileSize { get; set; }

		[DataMember(Name = "ContentType")]
		public string ContentType { get; set; }

		[DataMember(Name = "Thumbnail")]
		public Thumbnail Thumbnail { get; set; }
	}

	[DataContract]
	public class VideoResult
	{
		[DataMember(Name = "ID")]
		public Guid ID { get; set; }

		[DataMember(Name = "Title")]
		public string Title { get; set; }

		[DataMember(Name = "MediaUrl")]
		public string MediaUrl { get; set; }

		[DataMember(Name = "DisplayUrl")]
		public string DisplayUrl { get; set; }

		[DataMember(Name = "RunTime")]
		public int RunTime { get; set; }

		[DataMember(Name = "Thumbnail")]
		public Thumbnail Thumbnail { get; set; }
	}

	[DataContract]
	public class Thumbnail
	{
		[DataMember(Name = "MediaUrl")]
		public string MediaUrl { get; set; }

		[DataMember(Name = "ContentType")]
		public string ContentType { get; set; }

		[DataMember(Name = "Width")]
		public int Width { get; set; }

		[DataMember(Name = "Height")]
		public int Height { get; set; }

		[DataMember(Name = "FileSize")]
		public long FileSize { get; set; }
	}
	[DataContract]
	public class NewsResult
	{
		[DataMember(Name = "ID")]
		public Guid ID { get; set; }

		[DataMember(Name = "Title")]
		public string Title { get; set; }

		[DataMember(Name = "Source")]
		public string Source { get; set; }

		[DataMember(Name = "Description")]
		public string Description { get; set; }

		[DataMember(Name = "Url")]
		public string Url { get; set; }
	}

	[DataContract]
	public class RelatedSearchResult
	{
		[DataMember(Name = "ID")]
		public Guid ID { get; set; }

		[DataMember(Name = "Title")]
		public string Title { get; set; }

		[DataMember(Name = "BingUrl")]
		public string BingUrl { get; set; }
	}

	[DataContract]
	public class SpellResult
	{
		[DataMember(Name = "ID")]
		public Guid ID { get; set; }

		[DataMember(Name = "Value")]
		public string Value { get; set; }
	}
}
